unit lt3n_convert_old_basic_u1;

{$mode objfpc}{$H+}

//See
//http://sheepdogguides.com/lut/lt3n-convert-old-basic.htm
//for discussion.

//Program to illustrate some techniques which are
//generally useable, and particularly relevant
//to adapting old "console" apps to modern event
//driven operating systems.

//"States" and the timer event both play roles,
//as do sundry "flag" variables.

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, StdCtrls,
  ExtCtrls;

const kVers='17 Nov 18- 08-10';
  kPrgmName='lt3n_convert_BASIC';

  kRogue=-1234.5678;//Rogue value a user
    //"couldn't" answer to the questions

type

  { Tlt3n_convert_old_basic_f1 }

  Tlt3n_convert_old_basic_f1 = class(TForm)
    eUserInput: TEdit;
    laCurrentStateID: TLabel;
    laTermsDone: TLabel;
    laTxtCurrentStateID: TLabel;
    laTxtNotNormallyVisible: TLabel;
    laTxtTermsDone: TLabel;
    laTxtUserInputUse: TLabel;
    meShowUser: TMemo;
    Panel1: TPanel;
    Timer1: TTimer;
    procedure eUserInputEditingDone(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
  private
    { private declarations }
    bState,bTermsDone,bAns:byte;
    boInputReceived:boolean;
    bNextTerm:byte;
    sNextOpIs:string;
    extUserEntered:extended;
    procedure MakeState(bNewState:byte);
  public
    { public declarations }
  end;

var
  lt3n_convert_old_basic_f1: Tlt3n_convert_old_basic_f1;

implementation

{$R *.lfm}

{ Tlt3n_convert_old_basic_f1 }

(*Overview...

"in state x" defined by value in bState.

FormCreate will normally leave the application
in state 0.

State 0: Start problem.
We enter it from: The FormCreate handler
We go from it to: State 1. While in state 0, we put
a "Welcome" message up, and we post the first part
of the problem, e.g. "What is 5+4?".
We set bTermsDone to 2, because when we leave,
the user will have done 2 terms of the problem.
We leave the state when some input has been seen.

State 1: Process input. We enter it from: State 0 the
first time, and subsequently from state 2
We go from it to: 2 if a right answer has been given,
or we stay in 1, or we go on to state 3 if
enough terms have been presented and answered correctly

State 2: Present next term. We enter it from: State 1
We go from it to: State 1

State 3: Wrap up, after last part of question done.
We enter it from: state 1
We go nowhere from it.. it contains the end of the application.
*)

procedure Tlt3n_convert_old_basic_f1.FormCreate(Sender: TObject);
begin
   lt3n_convert_old_basic_f1.caption:=
     kPrgmName+' vers:'+kVers;

   meShowUser.lines.clear;

   Timer1.interval:=100;
   //NO??? Timer1.enabled:=false;//Don't start it until first
     //pass through the DoState0Things handler.

   eUserInput.text:='';
   Panel1.caption:='';
   laTermsDone.caption:='';

   MakeState(0);//Initialize bState, display


   laTxtUserInputUse.caption:='Press the Enter key after'+
   chr(13)+'any answers are entered,'+
   chr(13)+'to say "answer complete"';

   boInputReceived:=false;//Initialize

end;

procedure Tlt3n_convert_old_basic_f1.eUserInputEditingDone(Sender: TObject);
var boTmp:boolean;
begin
  boInputReceived:=true;

  //=== Start of slighly tedious matter ----
  //Try to turn what is in eUserInput.text, a STRING,
  //as far as the computer is concerned... could be
  // "xyz"... into a number AS FAR AS THE COMPUTER
  //is concerned. (The STRING "27" does not "equal"
  //the NUMBER 27, in the computer. And in a moment
  //we want to compare what the user entered to the
  //NUMBER he/she SHOULD have entered. (Edit boxes
  //can accept almost anything... including "xyz",
  //so provision must be made for the fact that the
  //user might have entered "xyz". It would be a
  //nonsense answer, but humans do generate nonsense
  //from time to time.

  boTmp:=TryStrToFloat(eUserInput.text,extUserEntered);
  if (not boTmp) then extUserEntered:=kRogue;

  //At this point, extUserEntered
  //  holds what the user entered in the edit box,
  //  if that could be converted to a number.
  //If it could not, extUserEntered holds kRogue
  //--- End of "slightly tedious matter"===

  eUserInput.text:='';//Clear the edit box for the next
  //problem. (Happily, this does not trigger an
  //OnInputEditingDone event.)
end;

procedure Tlt3n_convert_old_basic_f1.Timer1Timer(Sender: TObject);

  procedure DoState0Things;//SR of Timer1Timer event handler.
  //State0: Start problem
  begin
    meShowUser.lines.add('Welcome!');
    meShowUser.lines.add('-------');
    meShowUser.lines.add('');
    meShowUser.lines.add('What is 5+4?');
    bAns:=9;
    bTermsDone:=2;
    MakeState(1);
  end;//DoState0Things

  procedure DoState1Things;//SR of Timer1Timer event handler.
  //State1 ProcessInput
  begin
   if boInputReceived then begin
   boInputReceived:=false;
     case bTermsDone of
      2:begin
        if (extUserEntered=bAns) then
          begin
           sNextOpIs:='.. add ';
           bNextTerm:=2;
           bAns:=11;
           MakeState(2);
           bTermsDone:=3;
         end;//if extUserEntered=bAns
        end;//Case 2
      3:begin
        if (extUserEntered=bAns) then
          begin
           sNextOpIs:='.. subtact ';
           bNextTerm:=5;
           bAns:=6;
           MakeState(2);
           bTermsDone:=4;
         end;//if extUserEntered=bAns
        end;//Case 3
      4:begin
        if (extUserEntered=bAns) then
          begin
           MakeState(3);
         end;//if extUserEntered=bAns
        end;//Case 4

     end;//case bTerms of...
   end;//If boInputReceived
  end;//DoState1Things

  procedure DoState2Things;//SR of Timer1Timer event handler.
  //State2: Present next term
  begin
   meShowUser.lines.add('Yes! Now...');
   meShowUser.lines.add(sNextOpIs+
           inttostr(bNextTerm)+'?');
   MakeState(1);
  end;//DoState2Things

  procedure DoState3Things;//SR of Timer1Timer event handler.
  //State3: Wrap up
  begin
   meShowUser.lines.add('Yes again...');
   meShowUser.lines.add('...and you are done!');
   Timer1.enabled:=false;
  end;//DoState3Things

  (*Boilerplate...
  procedure DoStateThings;//SR of Timer1Timer event handler.
  begin
  end;//DoState2Things
  *)

begin //main block of Timer1Timer
    //boInputReceived:=false;//restore to "waiting"
    case bState of
    0:begin
      DoState0Things;
      end;
    1:begin
      DoState1Things;
      end;
    2:begin
      DoState2Things;
      end;
    3:begin
      DoState3Things;
      end;
    end;//case
end;//Timer1Timer

procedure Tlt3n_convert_old_basic_f1.
  MakeState(bNewState:byte);
begin
  bState:=bNewState;
  laCurrentStateID.caption:=inttostr(bState);
  laTermsDone.caption:=inttostr(bTermsDone);
end;//MakeState

end.

